#!/bin/csh 
#
#
set a = '$1'
set b = '$2'
set c = '$3'
set title = `pwd`
echo "0.001 1" > dum.dat
echo "1  0" >> dum.dat
#
#
#****************************************************************
if ($#argv == 2) then
#****************************************************************
#
cat << EOF | gnuplot 
reset
#######################################################
#  2 plots (observed versus predicted plot)
#
#  postscript output
#
set term postscript landscape enhanced color solid "Palatino-BoldItalic" 14
set output 'coherence.ps'
#
set xlabel 'Wavenumber (km^{-1})'; set x2label 'Wavelength (km)'
set ylabel 'Coherence'
set key graph 0.2,0.8 Left box 3 ; set tics out

set logscale x2 10;set x2range [6283.1853:6.2831853] reverse; set x2tics border
set yrange [0:1]; set ytics border 0.1; set mytics 2
set xrange [0.001:1]; set logscale x 10; set xtics border nomirror; 
set title "$title" "Times-BoldItalic, 10"

plot '$argv[1]' using ($a):($b) axes x1y1 title "$argv[1]" with points pointtype 3 pointsize 2, \
'$argv[2]' using ($a):($b) axes x1y1 title "$argv[2]" with linespoints pointtype 6 ps 2 ,\
'dum.dat' using ((2*pi)/$a):($b) axes x2y1 notitle with points pointtype 0 pointsize -1
#
set term X11
!gv coherence.ps &
EOF
#
#****************************************************************
else if ($#argv == 3) then
#****************************************************************
#
#~~~~~~~~~~~~~~~
cat << EOF > findmin.awk
BEGIN{minimum=100}
 {if (\$2 < minimum) {
  minimum=\$2 
  tevalue=\$1
  }
  } 
END{printf("%E\t%f\n", minimum, tevalue)}
EOF
#
set value=`gawk -f findmin.awk $argv[1]`
rm findmin.awk
echo "Best fit Te value is " $value[2]
#~~~~~~~~~~~~~~~
#
cat << EOF | gnuplot 
reset
#######################################################
#   multiplot of data
#
#  postscript output
#
set term postscript portrait enhanced color solid "Palatino-BoldItalic" 12
set output 'multi_coherence.ps'
#
set size 1.0, 1.0
set origin 0.0,0.0
set multiplot
#
set size 0.9,0.45
set origin 0.05,0.55
#
#  observed versus predicted plots
#
set xlabel 'Wavenumber (km^{-1})'; set x2label 'Wavelength (km)'
set ylabel 'Coherence'
set key graph 0.75,0.9 Left box 3; set tics out

set logscale x2 10;set x2range [6283.1853:6.2831853] reverse; set x2tics border
set yrange [0:1]; set ytics border 0.1; set mytics 2
set xrange [0.001:1]; set logscale x 10; set xtics border nomirror
set title "$title" "Times-BoldItalic, 10"

plot '$argv[1]' using ($a):($b) axes x1y1 title "$argv[1]" with points pointtype 3 pointsize 1, \
'$argv[2]' using ($a):($b) axes x1y1 title "$argv[2]" with linespoints pointtype 8 ps 1 ,\
'dum.dat' using ((2*pi)/$a):($b) axes x2y1 notitle with points pointtype 0 pointsize 0
#
set size 0.9,0.45
set origin 0.05,0.05
#
#  cop.dat plots
#
set yrange [*:*]; set ytics border 0.02; set mytics 4 
if ($value[2] <= 65) set xrange [0:100]; set xtics border 10; set xtics mirror; \
set mxtics 4; set x2range [0:100] noreverse
#
if ($value[2] > 65  && $value[2] <= 100 ) set xrange [0:150]; set xtics border 10; set xtics mirror; \
set mxtics 4; set x2range [0:100] noreverse
#
if ($value[2] > 100) set xrange [0:250]; set xtics border 20; set xtics mirror; \
set mxtics 4; set x2range [0:250] noreverse
set nologscale x; set nologscale x2 ; set nox2tics
set xlabel 'Elastic thickness (km)'; set x2label '  '
set ylabel 'Residual error'
set key graph 0.80,0.9 Left box 3
plot '$argv[3]' using ($a):($b) axes x1y1 title "L1-norm" with points pointtype 3 pointsize 2, \
'$argv[3]' using ($a):($c) axes x1y1 title "Error function" with linespoints pointtype 7 ps 1
#
set nomultiplot
set term X11
!gv multi_coherence.ps &
EOF
#
#****************************************************************
else if ($#argv == 1) then
#****************************************************************
#
#######################################################
#  1 plot (e.g. observed plot)
#
  echo "  "
  echo " Enter (C or c) for a coherence plot if the file is .obs OR .prd "
  echo " OR (E or e) for an error plot if the file is cop.dat"
  set answer = ($<)
  echo "  "
if ($answer == c || $answer == C) then
#
cat << EOF | gnuplot 
reset
#  postscript output
#
set term postscript landscape enhanced color solid "Palatino-BoldItalic" 14
set output 'coherence.ps'
#
set xlabel 'Wavenumber (km^{-1})'; set x2label 'Wavelength (km)'
set ylabel 'Coherence'
set key graph 0.2,0.8 Left box 3 ; set tics out

set logscale x2 10;set x2range [6283.1853:6.2831853] reverse; set x2tics border
set yrange [0:1]; set ytics border 0.1; set mytics 2
set xrange [0.001:1]; set logscale x 10; set xtics border nomirror; 
set title "$title" "Times-BoldItalic, 10"

plot '$argv[1]' using ($a):($b) axes x1y1 title "$argv[1]" with points pointtype 3 pointsize 2, \
'dum.dat' using ((2*pi)/$a):($b) axes x2y1 notitle with points pointtype 0 pointsize -1
#
set term X11
!gv coherence.ps &
EOF
##################
else if ($answer == e || $answer == E) then
##################
#
#~~~~~~~~~~~~~~~
cat << EOF > findmin.awk
BEGIN{minimum=100}
 {if (\$2 < minimum) {
  minimum=\$2 
  tevalue=\$1
  }
  } 
END{printf("%E\t%f\n", minimum, tevalue)}
EOF
#
set value=`gawk -f findmin.awk $argv[1]`
rm findmin.awk
echo "Best fit Te value is " $value[2]
#~~~~~~~~~~~~~~~
cat << EOF | gnuplot 
reset
#  postscript output
#
set term postscript landscape enhanced color solid "Palatino-BoldItalic" 14
set output 'error.ps'
#
set yrange [*:*]; set ytics border 0.02; set mytics 4
if ($value[2] <= 65) set xrange [0:100]; set xtics border 10; set mxtics 4
#
if ($value[2] > 65  && $value[2] <= 100 ) set xrange [0:150]; set xtics border 10; set mxtics 4
#
if ($value[2] > 100) set xrange [0:250]; set xtics border 20; set mxtics 4
set nologscale x
set xlabel 'Elastic thickness (km)'
set ylabel 'Residual error'
set key graph 0.85,0.85 Left box 3; set tics out
set title "$title" "Times-BoldItalic, 10"

plot '$argv[1]' using ($a):($b) title "L1-norm" with points pointtype 3 pointsize 2, \
'$argv[1]' using ($a):($c) title "Error Function" with linespoints pointtype 7 ps 1
#
set term X11
!gv error.ps &
EOF
##################
else 
##################
echo " "
echo " WRONG CHOICE !!!"
echo " Please try again !!"
echo " "
endif
#
#****************************************************************
else
#****************************************************************
#
  echo "  "
  echo "======================================================================"
  echo "  "
  echo " Usage: coplts.csh [arg1] [arg2] [arg3] "
  echo "  "
  echo " This program requires either 1, 2 or 3 arguments."
  echo "  "
  echo " This program uses gnuplot and gawk to generate postscript plots of \
  coherence and RMS error for Tony Lowry's MEM coherence codes."
  echo "  "
  echo " If 1 argument is given (eg. br.obs OR cop.dat) then a single \
  coherence plot or RMS error is generated.  You will be prompted to \
  enter C or E to choose either coherence or error plot generation \
  respectively."
  echo "  "
  echo " When 2 arguments are given (eg. br.obs AND br.32.5.prd) then the \
  observed versus best fit predicted coherence is plotted."
  echo "  "
  echo " When 3 arguments are given (eg. br.obs AND br.45.6.prd AND cop.dat) \
  then a stacked plot with observed vs. predicted coherence is plotted \
  above the RMS error plot."
  echo "  "
  echo "  "
  echo " Written by Gareth Armstrong, \
  Dept. Earth Sciences, Uni. Oxford, 1999 "
  echo "  "
  echo "======================================================================"
#
endif
rm dum.dat

