c prdmem.f ***** version: 09 Nov 2002 ******************************
c Estimates the maximum entropy auto- and cross-spectral densities
c of gridded two-dimensional Bouguer gravity and topographic data
c using the iterative technique developed
c in Lim and Malik [IEEE Trans. Acoustics, Speech, and Signal Proc.,
c ASSP-29, 401-413, 1981]. The program outputs the power spectrum
c for use in coherence analysis of flexural thickness.
c
c******************************************************************
c
c  Modified by Gareth Armstrong, Dept. Earth Sciences, Uni. Oxford,
c  on 17/2/99 for adjustable
c  array sizes using include files and to use the Salford ftn77 compiler
c  call to COMMAND_LINE to read in command line arguments similar to
c  the f77 compiler call to getarg when being used on the PC
c
c******************************************************************
c---------------------- main program ------------------------------
      implicit none
      include'coheren.inc'
c
c common variables
c
      real*8 alatmn,alatmx,alonmn,alonmx,alph,beta,ednom,ek,eps,
     &     rl(2),rwk(nxmax),spac,theta
      complex*16 drxx(ncmax,nrmax),dstr(ncmax,nrmax),dt(ncmax,nrmax), 
     &        dw(ncmax,nrmax),dwk(nxmax)
      integer ilat0,ilon0,iwk(nxmax),maxx,maxy,nc,nca,nr,nra
      common /ADIM  / nc,nr,rl,ilat0,ilon0,spac,theta,alonmn,
     &                alonmx,alatmn,alatmx
      common /FFT   / drxx,iwk,rwk,dwk
      common /PAR   / ek,alph,beta,eps,ednom,nca,nra,maxx,maxy
      common /STR   / dstr,dt,dw
c
c Local variables
c
      real*8 a,amnlamm,amnlwin,amnrwin,amntrp,betmn,eps0,epsold
      real*8 dabs
      character ans*1,infile*20,outfile*20,card*40
      integer ifl,ijob,ipsd,kk,m,maxit
c
      call getarg(1,infile)
      call getarg(2,outfile)
c
c-----------------------------------------------------------------------
c  THE NEXT call TO COMMAND_LINE PERTAINS TO THE SALFORD FTN77 COMPILER
c  AND IS USED ON THE PC INSTEAD OF f77'S call TO getarg
c-----------------------------------------------------------------------
c
c      call COMMAND_LINE(card)
c      read(card,*)infile,outfile
c
      open(13,file=infile,form='formatted',status='old')
      rewind(13)
      open(14,file=outfile,form='unformatted',status='unknown')
      rewind(14)
      open(15,file='track.dat',form='formatted',status='unknown')
      rewind(15)
c
      read(13,*)ijob
      read(13,*)ipsd
      read(13,*)maxit
      read(13,*)eps0
c
      if(ijob.eq.1)then
        call inputp(ipsd)
      elseif(ijob.eq.2)then
        call inputo(ipsd)
      else
        stop
      endif
c
      nca=nc
      nra=nr
c
      maxx=2
1     if(maxx.le.nca.or.maxx.le.nra)then
        maxx=2*maxx
        goto1
      endif
      maxx=2*maxx
      maxy=maxx
c
      read(13,*)ans
      if(ans.eq.'y'.or.ans.eq.'Y')then
        read(13,*)maxx,maxy
      endif
c
      if(ipsd.eq.1)then
        call crosscorr
      elseif(ipsd.eq.2)then
        call autocorrg
      elseif(ipsd.eq.3)then
        call autocorrt
      else
        stop
      endif
c
      kk=0
      call move
      call init
      amnlamm=dreal(drxx(1,1))
      call storetlm
      call inv
      amntrp=dreal(drxx(1,1))
      m=0
      do while(.true.)
        m=m+1
        call rearr
        call trnsfm(-1)
        call rearr
        epsold=eps
        call seteps
        if(m.eq.1)then
          ednom=ednom*eps
          eps=1d0
        endif
        write(15,*)m,eps,alph,beta
        if(eps.lt.eps0.or.kk.gt.0)then
          goto200
        endif
        if(eps.gt.epsold.or.beta.ne.0d0)then
          alph=(1d0+alph)/2d0
          ek=ek/2d0
        endif
        call storewindow
        call rearr
        call trnsfm(1)
        call rearr
        call min3(amnrwin,ifl)
        if(ifl.eq.1)then
          write(15,*)'Singularity encountered'
          goto200
        endif
        if(amnrwin.le.0d0)then
          a=1d0-ek*(amntrp/dabs(amnrwin))
          if(a.gt.alph)alph=a
        endif
        if(alph.gt.0.9d0)kk=kk+1
        call switch
        call rearr
        call trnsfm(1)
        call rearr
        call adda
        call inv
        call rearr
        call trnsfm(-1)
        call rearr
        call winstore
        call rearr
        call trnsfm(1)
        call rearr
        call min3(amnlwin,ifl)
        if(ifl.eq.1)then
          write(15,*)'Singularity encountered'
          goto200
        endif
        if(amnlwin.le.0d0)then
          betmn=dabs(amnlwin)/(amnlamm+dabs(amnlwin))
          beta=(1d0+(1d0-ek)*(1d0/betmn-1d0))*betmn
          alph=(1d0+alph)/2d0
        else
          beta=0d0
        endif
        call store
        call getlam
        call addb
        call min3(amnlamm,ifl)
        if(ifl.eq.1)then
          write(15,*)'Singularity encountered'
          goto200
        endif
        call storetlm
        call inv
        call min3(amntrp,ifl)
        if(ifl.eq.1)then
          write(15,*)'Singularity encountered'
          goto200
        endif
        call store
        if(m.eq.maxit)goto200
      enddo
 
200   call switch
      call rearr
      call trnsfm(-1)
      call rearr
      call setwinv
      call rearr
      call trnsfm(1)
      call switch
      call putv
      call rearr
      call trnsfm(1)
      call setwinu
      call switch
      call output(ijob)
c
      end
c
c--------------------- data input -----------------------------------
      subroutine inputp(ipsd)
      implicit none
      include'coheren.inc'
c
c common variables
c
      real*8 alatmn,alatmx,alonmn,alonmx,rl(2),spac,theta
      complex*16 dstr(ncmax,nrmax),dt(ncmax,nrmax),dw(ncmax,nrmax)
      integer ilat0,ilon0,nc,nr
      common /ADIM  / nc,nr,rl,ilat0,ilon0,spac,theta,alonmn,
     &                alonmx,alatmn,alatmx
      common /STR   / dstr,dt,dw
c
c Dummy arguments
c
      integer ipsd
c
c Local variables
c
      complex*16 dcmplx
      character*20 filini,filinr
      complex*16 s1
      integer i,id,j
      real*8 rowi(ncmax),rowr(ncmax)
c
c...  read in data by rows
      if(ipsd.eq.1.or.ipsd.eq.3)then
        read(13,*)filinr
        read(13,*)filini
        open(17,file=filinr,form='unformatted',status='old')
        rewind(17)
        open(18,file=filini,form='unformatted',status='old')
        rewind(18)
        read(17)id,ilon0,ilat0,spac,theta
        read(17)nc,nr,rl(1),rl(2)
        read(17)alonmn,alonmx,alatmn,alatmx
        read(18)id,ilon0,ilat0,spac,theta
        read(18)nc,nr,rl(1),rl(2)
        read(18)alonmn,alonmx,alatmn,alatmx
        s1=dcmplx(0d0)
        do j=1,nr
          read(17)(rowr(i),i=1,nc)
          read(18)(rowi(i),i=1,nc)
          do i=1,nc
            dt(i,j)=dcmplx(rowr(i),rowi(i))
            s1=s1+dt(i,j)
          enddo
        enddo
        s1=s1/dfloat(nc*nr)
        do j=1,nr
          do i=1,nc
            dt(i,j)=dt(i,j)-s1
          enddo
        enddo
      endif
      if(ipsd.eq.1.or.ipsd.eq.2)then
        read(13,*)filinr
        read(13,*)filini
        open(19,file=filinr,form='unformatted',status='old')
        rewind(19)
        open(20,file=filini,form='unformatted',status='old')
        rewind(20)
        read(19)id,ilon0,ilat0,spac,theta
        read(19)nc,nr,rl(1),rl(2)
        read(19)alonmn,alonmx,alatmn,alatmx
        read(20)id,ilon0,ilat0,spac,theta
        read(20)nc,nr,rl(1),rl(2)
        read(20)alonmn,alonmx,alatmn,alatmx
        s1=dcmplx(0d0)
        do j=1,nr
          read(19)(rowr(i),i=1,nc)
          read(20)(rowi(i),i=1,nc)
          do i=1,nc
            dw(i,j)=dcmplx(rowr(i),rowi(i))
            s1=s1+dw(i,j)
          enddo
        enddo
        s1=s1/dfloat(nc*nr)
        do j=1,nr
          do i=1,nc
            dw(i,j)=dw(i,j)-s1
          enddo
        enddo
      endif
c
      return
      end
c
c--------------------- data input -----------------------------------
      subroutine inputo(ipsd)
      implicit none
      include'coheren.inc'
c
c common variables
c
      real*8 alatmn,alatmx,alonmn,alonmx,rl(2),spac,theta
      complex*16 dstr(ncmax,nrmax),dt(ncmax,nrmax),dw(ncmax,nrmax)
      integer ilat0,ilon0,nc,nr
      common /ADIM  / nc,nr,rl,ilat0,ilon0,spac,theta,alonmn,
     &                alonmx,alatmn,alatmx
      common /STR   / dstr,dt,dw
c
c Dummy arguments
c
      integer ipsd
c
c Local variables
c
      character*20 filing,filint
      complex*16 s1
      integer i,id,j
      real*8 rd(512)
c
c...  read in data by rows
      if(ipsd.eq.1.or.ipsd.eq.3)then
        read(13,*)filint
        open(17,file=filint,form='unformatted',status='old')
        rewind(17)
        read(17)id,ilon0,ilat0,spac,theta
        if(id.ne.1)then
          stop
        endif
        read(17)nc,nr,rl(1),rl(2)
        read(17)alonmn,alonmx,alatmn,alatmx
        s1=dcmplx(0d0)
        do j=1,nr
          read(17)(rd(i),i=1,nc)
          do i=1,nc
            dt(i,j)=dcmplx(rd(i))
            s1=s1+dt(i,j)
          enddo
        enddo
        s1=s1/dfloat(nc*nr)
        do j=1,nr
          do i=1,nc
            dt(i,j)=dt(i,j)-s1
          enddo
        enddo
      endif
      if(ipsd.eq.1.or.ipsd.eq.2)then
        read(13,*)filing
        open(18,file=filing,form='unformatted',status='old')
        rewind(18)
        read(18)id,ilon0,ilat0,spac,theta
        read(18)nc,nr,rl(1),rl(2)
        read(18)alonmn,alonmx,alatmn,alatmx
        s1=dcmplx(0d0)
        do j=1,nr
          read(18)(rd(i),i=1,nc)
          do i=1,nc
            dw(i,j)=dcmplx(rd(i))
            s1=s1+dw(i,j)
          enddo
        enddo
        s1=s1/dfloat(nc*nr)
        do j=1,nr
          do i=1,nc
            dw(i,j)=dw(i,j)-s1
          enddo
        enddo
      endif
c
      return
      end
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine crosscorr
      implicit none
      include'coheren.inc'
c
c common variables
c
      real*8 alatmn,alatmx,alonmn,alonmx,alph,beta,ednom,ek,eps,
     &     rl(2),rwk(nxmax),spac,theta
      complex*16 drxx(ncmax,nrmax),dstr(ncmax,nrmax),dt(ncmax,nrmax),
     &        dw(ncmax,nrmax),dwk(nxmax)
      integer ilat0,ilon0,iwk(nxmax),maxx,maxy,nc,nca,nr,nra
      common /ADIM  / nc,nr,rl,ilat0,ilon0,spac,theta,alonmn,
     &                alonmx,alatmn,alatmx
      common /FFT   / drxx,iwk,rwk,dwk
      common /PAR   / ek,alph,beta,eps,ednom,nca,nra,maxx,maxy
      common /STR   / dstr,dt,dw
c
c Local variables
c
      complex*16 ddd
      complex*16 dcmplx,dconjg
      integer i,il,is,j,jl,js,k,l,midx,midy
      integer max0,min0
      real*8 acfsum
      real*8 dfloat,dreal
c
      midx=maxx/2+1
      midy=maxy/2+1
      do l=1-nra,nra-1
        do k=1-nca,nca-1
          acfsum=0d0
          js=max0(1,1-l)
          jl=min0(nr,nr-l)
          is=max0(1,1-k)
          il=min0(nc,nc-k)
          do j=js,jl
            do i=is,il
              acfsum=acfsum+dreal(dw(i,j)*dconjg(dt(i+k,j+l)))
            enddo
          enddo
          drxx(midx+k,midy+l)=dcmplx(acfsum)/dfloat(2*nc*nr)
        enddo
      enddo
      ddd=drxx(midx,midy)
      do j=1,maxy
        do i=1,maxx
          drxx(i,j)=drxx(i,j)/ddd
        enddo
      enddo
      return
      end
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine autocorrg
      implicit none
      include'coheren.inc'
c
c common variables
c
      real*8 alatmn,alatmx,alonmn,alonmx,alph,beta,ednom,ek,eps,
     &     rl(2),rwk(nxmax),spac,theta
      complex*16 drxx(ncmax,nrmax),dstr(ncmax,nrmax),dt(ncmax,nrmax),
     &         dw(ncmax,nrmax),dwk(nxmax)
      integer ilat0,ilon0,iwk(nxmax),maxx,maxy,nc,nca,nr,nra
      common /ADIM  / nc,nr,rl,ilat0,ilon0,spac,theta,alonmn,
     &                alonmx,alatmn,alatmx
      common /FFT   / drxx,iwk,rwk,dwk
      common /PAR   / ek,alph,beta,eps,ednom,nca,nra,maxx,maxy
      common /STR   / dstr,dt,dw
c
c Local variables
c
      complex*16 ddd
      complex*16 dcmplx,dconjg
      integer i,il,is,j,jl,js,k,l,midx,midy
      integer max0,min0
      real*8 acfsum
      real*8 dfloat,dreal
c
      midx=maxx/2+1
      midy=maxy/2+1
      do l=1-nra,0
        do k=1-nca,nca-1
          acfsum=0d0
          js=max0(1,1-l)
          jl=min0(nr,nr-l)
          is=max0(1,1-k)
          il=min0(nc,nc-k)
          do j=js,jl
            do i=is,il
              acfsum=acfsum+dreal(dw(i,j)*dconjg(dw(i+k,j+l)))
            enddo
          enddo
          drxx(midx+k,midy+l)=dcmplx(acfsum)/dfloat(nc*nr)
        enddo
      enddo
      do l=1,nra-1
        do k=1-nca,nca-1
          drxx(midx+k,midy+l)=dconjg(drxx(midx-k,midy-l))
        enddo
      enddo
      ddd=drxx(midx,midy)
      do j=1,maxy
        do i=1,maxx
          drxx(i,j)=drxx(i,j)/ddd
        enddo
      enddo
      return
      end
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine autocorrt
      implicit none
      include'coheren.inc'
c
c common variables
c
      real*8 alatmn,alatmx,alonmn,alonmx,alph,beta,ednom,ek,eps,
     &     rl(2),rwk(nxmax),spac,theta
      complex*16 drxx(ncmax,nrmax),dstr(ncmax,nrmax),dt(ncmax,nrmax),
     &        dw(ncmax,nrmax),dwk(nxmax)
      integer ilat0,ilon0,iwk(nxmax),maxx,maxy,nc,nca,nr,nra
      common /ADIM  / nc,nr,rl,ilat0,ilon0,spac,theta,alonmn,
     &                alonmx,alatmn,alatmx
      common /FFT   / drxx,iwk,rwk,dwk
      common /PAR   / ek,alph,beta,eps,ednom,nca,nra,maxx,maxy
      common /STR   / dstr,dt,dw
c
c Local variables
c
      complex*16 ddd
      complex*16 dcmplx,dconjg
      integer i,il,is,j,jl,js,k,l,midx,midy
      integer max0,min0
      real*8 acfsum
      real*8 dfloat,dreal
c
      midx=maxx/2+1
      midy=maxy/2+1
      do l=1-nra,0
        do k=1-nca,nca-1
          acfsum=0d0
          js=max0(1,1-l)
          jl=min0(nr,nr-l)
          is=max0(1,1-k)
          il=min0(nc,nc-k)
          do j=js,jl
            do i=is,il
              acfsum=acfsum+dreal(dt(i,j)*dconjg(dt(i+k,j+l)))
            enddo
          enddo
          drxx(midx+k,midy+l)=dcmplx(acfsum)/dfloat(nc*nr)
        enddo
      enddo
      do l=1,nra-1
        do k=1-nca,nca-1
          drxx(midx+k,midy+l)=dconjg(drxx(midx-k,midy-l))
        enddo
      enddo
      ddd=drxx(midx,midy)
      do j=1,maxy
        do i=1,maxx
          drxx(i,j)=drxx(i,j)/ddd
        enddo
      enddo
      return
      end
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine move
      implicit none
      include'coheren.inc'
c
c common variables
c
      real*8 alph,beta,ednom,ek,eps,rwk(nxmax)
      complex*16 drxx(ncmax,nrmax),dstr(ncmax,nrmax),dt(ncmax,nrmax),
     &        dw(ncmax,nrmax),dwk(nxmax)
      integer iwk(nxmax),maxx,maxy,nca,nra
      common /FFT   / drxx,iwk,rwk,dwk
      common /PAR   / ek,alph,beta,eps,ednom,nca,nra,maxx,maxy
      common /STR   / dstr,dt,dw
c
c Local variables
c
      complex*16 dcm
      complex*16 dcmplx
      integer i,j
c
      dcm=dcmplx(0d0)
      do j=1,maxy
        do i=1,maxx
          dt(i,j)=dcm
          dw(i,j)=dcm
        enddo
      enddo
      do j=1,maxy
        do i=1,maxx
          dw(i,j)=drxx(i,j)
        enddo
      enddo
      return
      end
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine init
      implicit none
      include'coheren.inc'
c
c common variables
c
      real*8 alph,beta,ednom,ek,eps,rwk(nxmax)
      complex*16 drxx(ncmax,nrmax),dstr(ncmax,nrmax),dt(ncmax,nrmax),
     &        dw(ncmax,nrmax),dwk(nxmax)
      integer iwk(nxmax),maxx,maxy,nca,nra
      common /FFT   / drxx,iwk,rwk,dwk
      common /PAR   / ek,alph,beta,eps,ednom,nca,nra,maxx,maxy
      common /STR   / dstr,dt,dw
c
c Local variables
c
      complex*16 dtmp
      integer i,j,midx,midy
c
      midx=maxx/2+1
      midy=maxy/2+1
      ek=0.5d0
      alph=0d0
      beta=0d0
      eps=1d0
      ednom=0d0
      do j=1-nra,nra-1
        do i=1-nca,nca-1
          ednom=ednom+dw(i+midx,j+midy)*dw(i+midx,j+midy)
        enddo
      enddo
      dtmp=1d0/dw(midx,midy)
      do j=1,maxy
        do i=1,maxx
          drxx(i,j)=dtmp
        enddo
      enddo
      return
      end
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine min3(amin,ifl)
      implicit none
      include'coheren.inc'
c
c common variables
c
      real*8 alph,beta,ednom,ek,eps,rwk(nxmax)
      complex*16 drxx(ncmax,nrmax),dwk(nxmax)
      integer iwk(nxmax),maxx,maxy,nca,nra
      common /FFT   / drxx,iwk,rwk,dwk
      common /PAR   / ek,alph,beta,eps,ednom,nca,nra,maxx,maxy
c
c Dummy arguments
c
      real*8 amin
      integer ifl
c
c Local variables
c
      real*8 a
      real*8 dmin1,dreal
      integer i,j
c
      amin=dreal(drxx(1,1))
      ifl=0
      do j=1,maxy
        do i=1,maxx
          a=dreal(drxx(i,j))
          amin=dmin1(amin,a)
c         if(isnan(a))then
c           ifl=1
c         endif
        enddo
      enddo
      return
      end
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine store
      implicit none
      include'coheren.inc'
c
c common variables
c
      real*8 alph,beta,ednom,ek,eps,rwk(nxmax)
      complex*16 drxx(ncmax,nrmax),dstr(ncmax,nrmax),dt(ncmax,nrmax),
     &        dw(ncmax,nrmax),dwk(nxmax)
      integer iwk(nxmax),maxx,maxy,nca,nra
      common /FFT   / drxx,iwk,rwk,dwk
      common /PAR   / ek,alph,beta,eps,ednom,nca,nra,maxx,maxy
      common /STR   / dstr,dt,dw
c
c Local variables
c
      integer i,j
c
      do j=1,maxy
        do i=1,maxx
          dstr(i,j)=drxx(i,j)
        enddo
      enddo
      return
      end
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine storetlm
      implicit none
      include'coheren.inc'
c
c common variables
c
      real*8 alph,beta,ednom,ek,eps,rwk(nxmax)
      complex*16 drxx(ncmax,nrmax),dstr(ncmax,nrmax),dt(ncmax,nrmax),
     &        dw(ncmax,nrmax),dwk(nxmax)
      integer iwk(nxmax),maxx,maxy,nca,nra
      common /FFT   / drxx,iwk,rwk,dwk
      common /PAR   / ek,alph,beta,eps,ednom,nca,nra,maxx,maxy
      common /STR   / dstr,dt,dw
c
c Local variables
c
      integer i,j
c
      do j=1,maxy
        do i=1,maxx
          dt(i,j)=drxx(i,j)
        enddo
      enddo
      return
      end
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine getlam
      implicit none
      include'coheren.inc'
c
c common variables
c
      real*8 alph,beta,ednom,ek,eps,rwk(nxmax)
      complex*16 drxx(ncmax,nrmax),dstr(ncmax,nrmax),dt(ncmax,nrmax),
     &        dw(ncmax,nrmax),dwk(nxmax)
      integer iwk(nxmax),maxx,maxy,nca,nra
      common /FFT   / drxx,iwk,rwk,dwk
      common /PAR   / ek,alph,beta,eps,ednom,nca,nra,maxx,maxy
      common /STR   / dstr,dt,dw
c
c Local variables
c
      integer i,j
c
      do j=1,maxy
        do i=1,maxx
          drxx(i,j)=dt(i,j)
        enddo
      enddo
      return
      end
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine inv
      implicit none
      include'coheren.inc'
c
c common variables
c
      real*8 alph,beta,ednom,ek,eps,rwk(nxmax)
      complex*16 drxx(ncmax,nrmax),dwk(nxmax)
      integer iwk(nxmax),maxx,maxy,nca,nra
      common /FFT   / drxx,iwk,rwk,dwk
      common /PAR   / ek,alph,beta,eps,ednom,nca,nra,maxx,maxy
c
c Local variables
c
      integer i,j
c
      do j=1,maxy
        do i=1,maxx
          drxx(i,j)=1d0/drxx(i,j)
        enddo
      enddo
      return
      end
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine seteps
      implicit none
      include'coheren.inc'
c
c common variables
c
      real*8 alph,beta,ednom,ek,eps,rwk(nxmax)
      complex*16 drxx(ncmax,nrmax),dstr(ncmax,nrmax),dt(ncmax,nrmax),
     &        dw(ncmax,nrmax),dwk(nxmax)
      integer iwk(nxmax),maxx,maxy,nca,nra
      common /FFT   / drxx,iwk,rwk,dwk
      common /PAR   / ek,alph,beta,eps,ednom,nca,nra,maxx,maxy
      common /STR   / dstr,dt,dw
c
c Local variables
c
      integer i,j,midx,midy
      real*8 dreal
c
      midx=maxx/2+1
      midy=maxy/2+1
      eps=0d0
      do j=1-nra,nra-1
        do i=1-nca,nca-1
          eps=eps+dreal(dw(i+midx,j+midy)-(drxx(i+midx,j+midy)))**2
        enddo
      enddo
      eps=eps/ednom
      return
      end
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine storewindow
      implicit none
      include'coheren.inc'
c
c common variables
c
      real*8 alph,beta,ednom,ek,eps,rwk(nxmax)
      complex*16 drxx(ncmax,nrmax),dstr(ncmax,nrmax),dt(ncmax,nrmax),
     &        dw(ncmax,nrmax),dwk(nxmax)
      integer iwk(nxmax),maxx,maxy,nca,nra
      common /FFT   / drxx,iwk,rwk,dwk
      common /PAR   / ek,alph,beta,eps,ednom,nca,nra,maxx,maxy
      common /STR   / dstr,dt,dw
c
c Local variables
c
      complex*16 dcm
      complex*16 dcmplx
      integer i,j,midx,midy
c
      midx=maxx/2+1
      midy=maxy/2+1
      dcm=dcmplx(0d0)
      do j=1,maxy
        do i=1,maxx
          dstr(i,j)=drxx(i,j)
          drxx(i,j)=dcm
        enddo
      enddo
      do j=1-nra,nra-1
        do i=1-nca,nca-1
          drxx(midx+i,midy+j)=dw(midx+i,midy+j)-dstr(midx+i,midy+j)
        enddo
      enddo
      return
      end
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine setwinv
      implicit none
      include'coheren.inc'
c
c common variables
c
      real*8 alph,beta,ednom,ek,eps,rwk(nxmax)
      complex*16 drxx(ncmax,nrmax),dstr(ncmax,nrmax),dt(ncmax,nrmax),
     &        dw(ncmax,nrmax),dwk(nxmax)
      integer iwk(nxmax),maxx,maxy,nca,nra
      common /FFT   / drxx,iwk,rwk,dwk
      common /PAR   / ek,alph,beta,eps,ednom,nca,nra,maxx,maxy
      common /STR   / dstr,dt,dw
c
c Local variables
c
      integer i,j,mx,my
      real*8 pi,pix2,piy2,wv,x,y
c
      mx=maxx/2+1
      my=maxy/2+1
      pi=3.14159265358979d0
      pix2=.5d0*pi/nca
      piy2=.5d0*pi/nra
      do j=1-nra,nra-1
        y=j*piy2
        do i=1-nca,nca-1
          x=i*pix2
          wv=dcos(x)*dcos(y)
          drxx(mx+i,my+j)=(1d0-wv)*drxx(mx+i,my+j)+wv*dw(mx+i,my+j)
          if(cdabs(drxx(mx+i,my+j)).gt.cdabs(dw(mx+i,my+j)))
     &        drxx(mx+i,my+j)=dw(mx+i,my+j)
        enddo
      enddo
      return
      end
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine setwinu
      implicit none
      include'coheren.inc'
c
c common variables
c
      real*8 alph,beta,ednom,ek,eps,rwk(nxmax)
      complex*16 drxx(ncmax,nrmax),dstr(ncmax,nrmax),dt(ncmax,nrmax),
     &        dw(ncmax,nrmax),dwk(nxmax)
      integer iwk(nxmax),maxx,maxy,nca,nra
      common /FFT   / drxx,iwk,rwk,dwk
      common /PAR   / ek,alph,beta,eps,ednom,nca,nra,maxx,maxy
      common /STR   / dstr,dt,dw
c
c Local variables
c
      integer i,j,mx,my
      real*8 pi,pix2,piy2,wv,x,y
c
      mx=maxx/2+1
      my=maxy/2+1
      pi=3.14159265358979d0
      pix2=.5d0*pi/nca
      piy2=.5d0*pi/nra
      do j=1-nra,nra-1
        y=j*piy2
        do i=1-nca,nca-1
          x=i*pix2
          wv=dcos(x)*dcos(y)
          drxx(mx+i,my+j)=(1d0-wv)*dstr(mx+i,my+j)+wv*drxx(mx+i,my+j)
        enddo
      enddo
      return
      end
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine winstore
      implicit none
      include'coheren.inc'
c
c common variables
c
      real*8 alph,beta,ednom,ek,eps,rwk(nxmax)
      complex*16 drxx(ncmax,nrmax),dstr(ncmax,nrmax),dt(ncmax,nrmax),
     &        dw(ncmax,nrmax),dwk(nxmax)
      integer iwk(nxmax),maxx,maxy,nca,nra
      common /FFT   / drxx,iwk,rwk,dwk
      common /PAR   / ek,alph,beta,eps,ednom,nca,nra,maxx,maxy
      common /STR   / dstr,dt,dw
c
c Local variables
c
      complex*16 dcmplx
      complex*16 dcm
      integer i,j,midx,midy
c
      midx=maxx/2+1
      midy=maxy/2+1
      dcm=dcmplx(0d0)
      do j=1,maxy
        do i=1,maxx
          dstr(i,j)=drxx(i,j)
          drxx(i,j)=dcm
        enddo
      enddo
      do j=1-nra,nra-1
        do i=1-nca,nca-1
          drxx(midx+i,midy+j)=dstr(midx+i,midy+j)
        enddo
      enddo
      return
      end
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine putv
      implicit none
      include'coheren.inc'
c
c common variables
c
      real*8 alph,beta,ednom,ek,eps,rwk(nxmax)
      complex*16 drxx(ncmax,nrmax),dstr(ncmax,nrmax),dt(ncmax,nrmax),
     &        dw(ncmax,nrmax),dwk(nxmax)
      integer iwk(nxmax),maxx,maxy,nca,nra
      common /FFT   / drxx,iwk,rwk,dwk
      common /PAR   / ek,alph,beta,eps,ednom,nca,nra,maxx,maxy
      common /STR   / dstr,dt,dw
c
c Local variables
c
      complex*16 dcmplx
      complex*16 dcm
      integer i,j,midx,midy
c
      midx=maxx/2+1
      midy=maxy/2+1
      dcm=dcmplx(0d0)
      do j=1,maxy
        do i=1,maxx
          drxx(i,j)=dcm
        enddo
      enddo
      do j=1-nra,nra-1
        do i=1-nca,nca-1
          drxx(midx+i,midy+j)=dw(midx+i,midy+j)
        enddo
      enddo
      return
      end
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine switch
      implicit none
      include'coheren.inc'
c
c common variables
c
      real*8 alph,beta,ednom,ek,eps,rwk(nxmax)
      complex*16 drxx(ncmax,nrmax),dstr(ncmax,nrmax),dt(ncmax,nrmax),
     &        dw(ncmax,nrmax),dwk(nxmax)
      integer iwk(nxmax),maxx,maxy,nca,nra
      common /FFT   / drxx,iwk,rwk,dwk
      common /PAR   / ek,alph,beta,eps,ednom,nca,nra,maxx,maxy
      common /STR   / dstr,dt,dw
c
c Local variables
c
      complex*16 dtemp
      integer i,j
c
      do j=1,maxy
        do i=1,maxx
          dtemp=drxx(i,j)
          drxx(i,j)=dstr(i,j)
          dstr(i,j)=dtemp
        enddo
      enddo
      return
      end
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine rearr
      implicit none
      include'coheren.inc'
c
c common variables
c
      real*8 alph,beta,ednom,ek,eps,rwk(nxmax)
      complex*16 drxx(ncmax,nrmax),dwk(nxmax)
      integer iwk(nxmax),maxx,maxy,nca,nra
      common /FFT   / drxx,iwk,rwk,dwk
      common /PAR   / ek,alph,beta,eps,ednom,nca,nra,maxx,maxy
c
c Local variables
c
      complex*16 dtemp
      integer i,j,midx,midy
c
      midx=maxx/2
      midy=maxy/2
      do j=1,midy
        do i=1,midx
          dtemp=drxx(i,j)
          drxx(i,j)=drxx(i+midx,j+midy)
          drxx(i+midx,j+midy)=dtemp
        enddo
      enddo
      do j=midy+1,maxy
        do i=1,midx
          dtemp=drxx(i,j)
          drxx(i,j)=drxx(i+midx,j-midy)
          drxx(i+midx,j-midy)=dtemp
        enddo
      enddo
      return
      end
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine adda
      implicit none
      include'coheren.inc'
c
c common variables
c
      real*8 alph,beta,ednom,ek,eps,rwk(nxmax)
      complex*16 drxx(ncmax,nrmax),dstr(ncmax,nrmax),dt(ncmax,nrmax),
     &        dw(ncmax,nrmax),dwk(nxmax)
      integer iwk(nxmax),maxx,maxy,nca,nra
      common /FFT   / drxx,iwk,rwk,dwk
      common /PAR   / ek,alph,beta,eps,ednom,nca,nra,maxx,maxy
      common /STR   / dstr,dt,dw
c
c Local variables
c
      real*8 alpm
      integer i,j
c
      alpm=1d0-alph
      do j=1,maxy
        do i=1,maxx
          drxx(i,j)=drxx(i,j)+alpm*dstr(i,j)
        enddo
      enddo
      return
      end
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine addb
      implicit none
      include'coheren.inc'
c
c common variables
c
      real*8 alph,beta,ednom,ek,eps,rwk(nxmax)
      complex*16 drxx(ncmax,nrmax),dstr(ncmax,nrmax),dt(ncmax,nrmax),
     &        dw(ncmax,nrmax),dwk(nxmax)
      integer iwk(nxmax),maxx,maxy,nca,nra
      common /FFT   / drxx,iwk,rwk,dwk
      common /PAR   / ek,alph,beta,eps,ednom,nca,nra,maxx,maxy
      common /STR   / dstr,dt,dw
c
c Local variables
c
      real*8 betm
      integer i,j
c
      betm=1d0-beta
      do j=1,maxy
        do i=1,maxx
          drxx(i,j)=beta*drxx(i,j)+betm*dstr(i,j)
        enddo
      enddo
      return
      end
c
c-------------------- trnsfm ---------------------------
      subroutine trnsfm(ift)
      implicit none
      include'coheren.inc'
c
c common variables
c
      real*8 alph,beta,ednom,ek,eps,rwk(nxmax)
      complex*16 drxx(ncmax,nrmax),dwk(nxmax)
      integer iwk(nxmax),maxx,maxy,nca,nra
      common /FFT   / drxx,iwk,rwk,dwk
      common /PAR   / ek,alph,beta,eps,ednom,nca,nra,maxx,maxy
c
c Dummy arguments
c
      integer ift
c
c Local variables
c
      complex*16 d12
      complex*16 dcmplx
      integer i,j
      real*8 r12
c
      call fft3d(ift)
c
      r12=dfloat(maxx*maxy)
      d12=dcmplx(r12)
      if(ift.eq.1)then
        do j=1,maxy
          do i=1,maxx
            drxx(i,j)=drxx(i,j)/d12
          enddo
        enddo
      elseif(ift.eq.-1)then
        do j=1,maxy
          do i=1,maxx
            drxx(i,j)=drxx(i,j)*d12
          enddo
        enddo
      endif
      return
      end
c
c------------------- data output --------------------------
      subroutine output(ijob)
      implicit none
      include'coheren.inc'
c
c common variables
c
      real*8 alatmn,alatmx,alonmn,alonmx,alph,beta,ednom,ek,eps,
     &     rl(2),spac,theta
      complex*16 dstr(ncmax,nrmax),dt(ncmax,nrmax),dw(ncmax,nrmax)
      integer ilat0,ilon0,maxx,maxy,nc,nca,nr,nra
      common /ADIM  / nc,nr,rl,ilat0,ilon0,spac,theta,alonmn,
     &                alonmx,alatmn,alatmx
      common /PAR   / ek,alph,beta,eps,ednom,nca,nra,maxx,maxy
      common /STR   / dstr,dt,dw
c
c Dummy arguments
c
      integer ijob
c
c Local variables
c
      integer i,j
c
      write(14)ijob,ilon0,ilat0,spac,theta
      write(14)maxx,maxy,rl(1),rl(2)
      write(14)alonmn,alonmx,alatmn,alatmx
      do j=1,maxy
        write(14)(dstr(i,j),i=1,maxx)
      enddo
      return
      end
c
c------------------------ fft3d ---------------------------
      subroutine fft3d(ift)
      implicit none
      include'coheren.inc'
c
c common variables
c
      real*8 alph,beta,ednom,ek,eps,rwk(nxmax)
      complex*16 drxx(ncmax,nrmax),dwk(nxmax)
      integer iwk(nxmax),maxx,maxy,nca,nra
      common /FFT   / drxx,iwk,rwk,dwk
      common /PAR   / ek,alph,beta,eps,ednom,nca,nra,maxx,maxy
c
c Dummy arguments
c
      integer ift
c
c Local variables
c
      complex*16 dconjg
      integer i,j,l,m
      real*8 r12
c
c                                  FIRST EXECUTABLE STATEMENT
      if(ift.le.0)then
c                                  invERSE TRANSform
        do j=1,maxy
          do i=1,maxx
            drxx(i,j)=dconjg(drxx(i,j))
          enddo
        enddo
      endif
c                                  TRANSform SECOND SUBSCRIPT
      do l=1,maxx
        do m=1,maxy
          dwk(m)=drxx(l,m)
        enddo
        call fftcc(maxy)
        do j=1,maxy
          drxx(l,j)=dwk(j)
        enddo
      enddo
c                                  TRANSform FIRST SUBSCRIPT
      do j=1,maxy
        do i=1,maxx
          dwk(i)=drxx(i,j)
        enddo
        call fftcc(maxx)
        do i=1,maxx
          drxx(i,j)=dwk(i)
        enddo
      enddo
      if(ift.le.0)then
        r12=dfloat(maxx*maxy)
        do j=1,maxy
          do i=1,maxx
            drxx(i,j)=dconjg(drxx(i,j))/r12
          enddo
        enddo
      endif
      return
      end
c
c-------------------------- fftcc --------------------------
      subroutine fftcc(n)
      implicit none
      include'coheren.inc'
c
c common variables
c
      complex*16 drxx(ncmax,nrmax),dwk(nxmax)
      integer iwk(nxmax)
      real*8 rwk(nxmax)
      common /FFT   / drxx,iwk,rwk,dwk
c
c Dummy arguments
c
      integer n
c
c Local variables
c
      real*8 a0,a1,a2,a3,a4,b0,b1,b2,b3,b4,c1,c2,c3,c30,cm,
     &     half,one,rad,s1,s2,s3,sm,two,z0(2),z1(2),z2(2),
     &     z3(2),z4(2),zero
      complex*16 ak2,za0,za1,za2,za3,za4
      complex*16 dcmplx
      integer i,iam,iap,ibm,ibp,ic,icc,icf,ick,id,idm1,ii,
     &        ija,ikb,ikt,ill,im,ird,isf,isk,isp,iss,ita,itb,
     &        j,ja,jf,jj,jk,k,k0,k1,k2,k3,ka,kb,kd2,kf,kh,
     &        kn,kt,ktp,l,l1,m,mm,mm1,mp
      integer iabs
c
c                                  SPECIFICATIONS FOR LOCAL VARIABLES
      equivalence(za0,z0(1))
      equivalence(za1,z1(1))
      equivalence(za2,z2(1))
      equivalence(za3,z3(1))
      equivalence(a0,z0(1))
      equivalence(b0,z0(2))
      equivalence(a1,z1(1))
      equivalence(b1,z1(2))
      equivalence(a2,z2(1))
      equivalence(b2,z2(2))
      equivalence(a3,z3(1))
      equivalence(b3,z3(2))
      equivalence(za4,z4(1))
      equivalence(z4(1),a4)
      equivalence(z4(2),b4)
      data rad/6.283185/,c30/.8660254/
      data zero,half,one,two/0d0,0.5d0,1d0,2d0/
c                                  FIRST EXECUTABLE STATEMENT
      if(n.eq.1)goto1200
      k=n
      m=0
      j=2
      jj=4
      jf=0
c                                  DETERMINE THE SQUARE FACTORS OF N
      iwk(1)=1
      do while(.true.)
        i=k/jj
        if(i*jj.ne.k)then
          j=j+2
          if(j.eq.4)j=3
          jj=j*j
          if(jj.gt.k)then
            kt=m
c                                  DETERMINE THE REMAINING FACTORS OF N
            j=2
            do while(.true.)
              i=k/j
              if(i*j.ne.k)then
                j=j+1
                if(j.ne.3)then
                  j=j+1
                  if(j.gt.k)then
                    k=iwk(m+1)
                    if(iwk(kt+1).gt.iwk(m+1))k=iwk(kt+1)
                    if(kt.gt.0)then
                      ktp=kt+2
                      do i=1,kt
                        j=ktp-i
                        m=m+1
                        iwk(m+1)=iwk(j)
                      enddo
                    endif
                    mp=m+1
                    ic=mp+1
                    id=ic+mp
                    ill=id+mp
                    ird=ill+mp+1
                    icc=ird+mp
                    iss=icc+mp
                    ick=iss+mp
                    isk=ick+k
                    icf=isk+k
                    isf=icf+k
                    iap=isf+k
                    kd2=(k-1)/2+1
                    ibp=iap+kd2
                    iam=ibp+kd2
                    ibm=iam+kd2
                    mm1=m-1
                    i=1
                    goto100
                  endif
                endif
              else
                m=m+1
                iwk(m+1)=j
                k=i
              endif
            enddo
          endif
        else
          m=m+1
          iwk(m+1)=j
          k=i
        endif
      enddo
100   l=mp-i
      j=ic-i
      iwk(ill+l)=0
      if((iwk(j-1)+iwk(j)).eq.4)iwk(ill+l)=1
      if(iwk(ill+l).ne.0)then
        i=i+1
        l=l-1
        iwk(ill+l)=0
      endif
      i=i+1
      if(i.le.mm1)goto100
      iwk(ill+1)=0
      iwk(ill+mp)=0
      iwk(ic)=1
      iwk(id)=n
      do j=1,m
        k=iwk(j+1)
        iwk(ic+j)=iwk(ic+j-1)*k
        iwk(id+j)=iwk(id+j-1)/k
        rwk(ird+j)=rad/iwk(ic+j)
        c1=rad/k
        if(k.gt.2)then
          rwk(icc+j)=dcos(c1)
          rwk(iss+j)=dsin(c1)
        endif
      enddo
      mm=m
      if(iwk(ill+m).eq.1)mm=m-1
      if(mm.gt.1)then
        sm=iwk(ic+mm-2)*rwk(ird+m)
        cm=dcos(sm)
        sm=dsin(sm)
      endif
      kb=0
      kn=n
      jj=0
      i=1
      c1=one
      s1=zero
      l1=1
200   if(iwk(ill+i+1).eq.1)then
        kf=4
        i=i+1
      else
        kf=iwk(i+1)
      endif
      isp=iwk(id+i)
      if(l1.ne.1)then
        s1=jj*rwk(ird+i)
        c1=dcos(s1)
        s1=dsin(s1)
      endif
c                                  FACTORS OF 2,3,AND 4 ARE
c                                  HANDLED SEPARATELY.
300   if(kf.gt.4)then
        jk=kf-1
        kh=jk/2
        k3=iwk(id+i-1)
        k0=kb+isp
        if(l1.ne.1)then
          k=jk-1
          rwk(icf+1)=c1
          rwk(isf+1)=s1
          do j=1,k
            rwk(icf+j+1)=rwk(icf+j)*c1-rwk(isf+j)*s1
            rwk(isf+j+1)=rwk(icf+j)*s1+rwk(isf+j)*c1
          enddo
        endif
        if(kf.ne.jf)then
          c2=rwk(icc+i)
          rwk(ick+1)=c2
          rwk(ick+jk)=c2
          s2=rwk(iss+i)
          rwk(isk+1)=s2
          rwk(isk+jk)=-s2
          do j=1,kh
            k=jk-j
            rwk(ick+k)=rwk(ick+j)*c2-rwk(isk+j)*s2
            rwk(ick+j+1)=rwk(ick+k)
            rwk(isk+j+1)=rwk(ick+j)*s2+rwk(isk+j)*c2
            rwk(isk+k)=-rwk(isk+j+1)
          enddo
        endif
        do while(.true.)
          k0=k0-1
          k1=k0
          k2=k0+k3
          za0=dwk(k0+1)
          a3=a0
          b3=b0
          do j=1,kh
            k1=k1+isp
            k2=k2-isp
            if(l1.eq.1)then
              za1=dwk(k1+1)
              za2=dwk(k2+1)
            else
              k=kf-j
              za4=dwk(k1+1)
              a1=a4*rwk(icf+j)-b4*rwk(isf+j)
              b1=a4*rwk(isf+j)+b4*rwk(icf+j)
              za4=dwk(k2+1)
              a2=a4*rwk(icf+k)-b4*rwk(isf+k)
              b2=a4*rwk(isf+k)+b4*rwk(icf+k)
            endif
            rwk(iap+j)=a1+a2
            rwk(iam+j)=a1-a2
            rwk(ibp+j)=b1+b2
            rwk(ibm+j)=b1-b2
            a3=a1+a2+a3
            b3=b1+b2+b3
          enddo
          dwk(k0+1)=dcmplx(a3,b3)
          k1=k0
          k2=k0+k3
          do j=1,kh
            k1=k1+isp
            k2=k2-isp
            jk=j
            a1=a0
            b1=b0
            a2=zero
            b2=zero
            do k=1,kh
              a1=a1+rwk(iap+k)*rwk(ick+jk)
              a2=a2+rwk(iam+k)*rwk(isk+jk)
              b1=b1+rwk(ibp+k)*rwk(ick+jk)
              b2=b2+rwk(ibm+k)*rwk(isk+jk)
              jk=jk+j
              if(jk.ge.kf)jk=jk-kf
            enddo
            dwk(k1+1)=dcmplx(a1-b2,b1+a2)
            dwk(k2+1)=dcmplx(a1+b2,b1-a2)
          enddo
          if(k0.le.kb)then
            jf=kf
            goto400
          endif
        enddo
      elseif(kf.eq.3)then
        if(l1.ne.1)then
          c2=c1*c1-s1*s1
          s2=two*c1*s1
        endif
        ja=kb+isp-1
        ka=ja+kb
        ikb=kb+1
        ija=ja+1
        do ii=ikb,ija
          k0=ka-ii+1
          k1=k0+isp
          k2=k1+isp
          za0=dwk(k0+1)
          if(l1.eq.1)then
            za1=dwk(k1+1)
            za2=dwk(k2+1)
          else
            za4=dwk(k1+1)
            a1=a4*c1-b4*s1
            b1=a4*s1+b4*c1
            za4=dwk(k2+1)
            a2=a4*c2-b4*s2
            b2=a4*s2+b4*c2
          endif
          dwk(k0+1)=dcmplx(a0+a1+a2,b0+b1+b2)
          a0=-half*(a1+a2)+a0
          a1=(a1-a2)*c30
          b0=-half*(b1+b2)+b0
          b1=(b1-b2)*c30
          dwk(k1+1)=dcmplx(a0-b1,b0+a1)
          dwk(k2+1)=dcmplx(a0+b1,b0-a1)
        enddo
      elseif(kf.eq.4)then
        if(l1.ne.1)then
          c2=c1*c1-s1*s1
          s2=two*c1*s1
          c3=c1*c2-s1*s2
          s3=s1*c2+c1*s2
        endif
        ja=kb+isp-1
        ka=ja+kb
        ikb=kb+1
        ija=ja+1
        do ii=ikb,ija
          k0=ka-ii+1
          k1=k0+isp
          k2=k1+isp
          k3=k2+isp
          za0=dwk(k0+1)
          if(l1.eq.1)then
            za1=dwk(k1+1)
            za2=dwk(k2+1)
            za3=dwk(k3+1)
          else
            za4=dwk(k1+1)
            a1=a4*c1-b4*s1
            b1=a4*s1+b4*c1
            za4=dwk(k2+1)
            a2=a4*c2-b4*s2
            b2=a4*s2+b4*c2
            za4=dwk(k3+1)
            a3=a4*c3-b4*s3
            b3=a4*s3+b4*c3
          endif
          dwk(k0+1)=dcmplx(a0+a2+a1+a3,b0+b2+b1+b3)
          dwk(k1+1)=dcmplx(a0+a2-a1-a3,b0+b2-b1-b3)
          dwk(k2+1)=dcmplx(a0-a2-b1+b3,b0-b2+a1-a3)
          dwk(k3+1)=dcmplx(a0-a2+b1-b3,b0-b2-a1+a3)
        enddo
      else
        k0=kb+isp
        k2=k0+isp
        if(l1.eq.1)then
          do while(.true.)
            k0=k0-1
            if(k0.lt.kb)goto400
            k2=k2-1
            ak2=dwk(k2+1)
            dwk(k2+1)=dwk(k0+1)-ak2
            dwk(k0+1)=dwk(k0+1)+ak2
          enddo
        else
          do while(.true.)
            k0=k0-1
            if(k0.lt.kb)goto400
            k2=k2-1
            za4=dwk(k2+1)
            a0=a4*c1-b4*s1
            b0=a4*s1+b4*c1
            dwk(k2+1)=dwk(k0+1)-za0
            dwk(k0+1)=dwk(k0+1)+za0
          enddo
        endif
      endif
400   if(i.ge.mm)then
        i=mm
        l1=0
        kb=iwk(id+i-1)+kb
        if(kb.ge.kn)then
          i=1
          ja=kt-1
          ka=ja+1
          if(ja.ge.1)then
            do ii=1,ja
              j=ka-ii
              iwk(j+1)=iwk(j+1)-1
              i=iwk(j+1)+i
            enddo
          endif
c                                  THE RESULT IS NOW PERMUTED TO
c                                  NORMAL ORDER.
          if(kt.le.0)goto700
          j=1
          i=0
          kb=0
        else
          do while(.true.)
            jj=iwk(ic+i-2)+jj
            if(jj.ge.iwk(ic+i-1))then
              i=i-1
              jj=jj-iwk(ic+i)
            elseif(i.ne.mm)then
              if(iwk(ill+i).eq.1)i=i+1
              goto200
            else
              c2=c1
              c1=cm*c1-sm*s1
              s1=sm*c2+cm*s1
              goto300
            endif
          enddo
        endif
      else
        i=i+1
        goto200
      endif
500   k2=iwk(id+j)+kb
      k3=k2
      jj=iwk(ic+j-1)
      jk=jj
      k0=kb+jj
      isp=iwk(ic+j)-jj
600   k=k0+jj
      do while(.true.)
        za4=dwk(k0+1)
        dwk(k0+1)=dwk(k2+1)
        dwk(k2+1)=za4
        k0=k0+1
        k2=k2+1
        if(k0.ge.k)then
          k0=k0+isp
          k2=k2+isp
          if(k0.lt.k3)goto600
          if(k0.ge.k3+isp)then
            k3=iwk(id+j)+k3
            if(k3-kb.lt.iwk(id+j-1))then
              k2=k3+jk
              jk=jk+jj
              k0=k3-iwk(id+j)+jk
              goto600
            elseif(j.ge.kt)then
              kb=k3
              if(i.le.0)then
                if(kb.ge.n)goto700
                j=1
                goto500
              else
                j=iwk(ill+i)
                i=i-1
                goto500
              endif
            else
              k=iwk(j+1)+i
              j=j+1
              do while(.true.)
                i=i+1
                iwk(ill+i)=j
                if(i.ge.k)goto500
              enddo
            endif
          else
            k0=k0-iwk(id+j)+jj
            goto600
          endif
        endif
      enddo
700   jk=iwk(ic+kt)
      isp=iwk(id+kt)
      m=m-kt
      kb=isp/jk-2
      if(kt.ge.m-1)goto1200
      ita=ill+kb+1
      itb=ita+jk
      idm1=id-1
      ikt=kt+1
      im=m+1
      do j=ikt,im
        iwk(idm1+j)=iwk(idm1+j)/jk
      enddo
      jj=0
      do j=1,kb
        k=kt
        do while(.true.)
          jj=iwk(id+k+1)+jj
          if(jj.lt.iwk(id+k))then
            iwk(ill+j)=jj
            if(jj.eq.j)iwk(ill+j)=-j
            goto800
          else
            jj=jj-iwk(id+k)
            k=k+1
          endif
        enddo
800   enddo
c                                  DETERMINE THE PERMUTATION CYCLES
c                                  OF LENGTH GREATER THAN OR EQUAL
c                                  TO TWO.
      do j=1,kb
        if(iwk(ill+j).gt.0)then
          k2=j
          do while(.true.)
            k2=iabs(iwk(ill+k2))
            if(k2.eq.j)goto900
            iwk(ill+k2)=-iwk(ill+k2)
          enddo
        endif
900   enddo
c                                  REORDER A FOLLOWING THE
c                                  PERMUTATION CYCLES
      i=0
      j=0
      kb=0
      kn=n
1000  do while(.true.)
        j=j+1
        if(iwk(ill+j).ge.0)then
          k=iwk(ill+j)
          k0=jk*k+kb
          do while(.true.)
            za4=dwk(k0+i+1)
            rwk(ita+i)=a4
            rwk(itb+i)=b4
            i=i+1
            if(i.ge.jk)then
              i=0
              goto1100
            endif
          enddo
        endif
      enddo
1100  k=-iwk(ill+k)
      jj=k0
      k0=jk*k+kb
      do while(.true.)
        dwk(jj+i+1)=dwk(k0+i+1)
        i=i+1
        if(i.ge.jk)then
          i=0
          if(k.ne.j)goto1100
          do while(.true.)
            dwk(k0+i+1)=dcmplx(rwk(ita+i),rwk(itb+i))
            i=i+1
            if(i.ge.jk)then
              i=0
              if(j.lt.k2)goto1000
              j=0
              kb=kb+isp
              if(kb.ge.kn)goto1200
              goto1000
            endif
          enddo
        endif
      enddo
1200  return
      end
